/*
 *  GuiLoader, a GuiXml loader library
 *  Copyright (c) 2006-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#ifndef GUILOADER_CONTAINERS_H
#define GUILOADER_CONTAINERS_H

#include "guiloader_impl.h"

typedef struct _Place Place;

struct _Place {
	int a, b, c, d;
};

// CrowChild -------------------------------------------------------------------

#define CROW_TYPE_CHILD (crow_child_get_type())
#define CROW_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_CHILD, CrowChild))
#define CROW_IS_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_CHILD))

typedef struct _CrowChild CrowChild;

struct _CrowChild {
	GObject parent;
	GtkWidget * widget;
	Place place;
};

GType crow_child_get_type(void);
void crow_child_init(CrowChild * child);

// CrowRowChild ----------------------------------------------------------------

#define CROW_TYPE_ROW_CHILD (crow_row_child_get_type())
#define CROW_ROW_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_ROW_CHILD, CrowRowChild))
#define CROW_IS_ROW_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_ROW_CHILD))

typedef struct _CrowChild CrowRowChild;

GType crow_row_child_get_type(void);

// CrowXYChild -----------------------------------------------------------------

#define CROW_TYPE_XY_CHILD (crow_xy_child_get_type())
#define CROW_XY_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_XY_CHILD, CrowXYChild))
#define CROW_IS_XY_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_XY_CHILD))

typedef struct _CrowChild CrowXYChild;

GType crow_xy_child_get_type(void);

// CrowBoxChild ----------------------------------------------------------------

#define CROW_TYPE_BOX_CHILD (crow_box_child_get_type())
#define CROW_BOX_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_BOX_CHILD, CrowBoxChild))
#define CROW_IS_BOX_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_BOX_CHILD))

typedef struct _CrowBoxChild CrowBoxChild;

struct _CrowBoxChild {
	CrowChild parent;
	CrowPackOptions options;
	int padding;
};

GType crow_box_child_get_type(void);
void crow_box_child_init(CrowBoxChild * child);

// CrowButtonBoxChild ----------------------------------------------------------

#define CROW_TYPE_BUTTON_BOX_CHILD (crow_button_box_child_get_type())
#define CROW_BUTTON_BOX_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_BUTTON_BOX_CHILD, CrowButtonBoxChild))
#define CROW_IS_BUTTON_BOX_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_BUTTON_BOX_CHILD))

typedef struct _CrowButtonBoxChild CrowButtonBoxChild;

struct _CrowButtonBoxChild {
	CrowBoxChild parent;
	gboolean secondary;
};

GType crow_button_box_child_get_type(void);
void crow_button_box_child_init(CrowButtonBoxChild * child);

// CrowPanedChild --------------------------------------------------------------

#define CROW_TYPE_PANED_CHILD (crow_paned_child_get_type())
#define CROW_PANED_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_PANED_CHILD, CrowPanedChild))
#define CROW_IS_PANED_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_PANED_CHILD))

typedef struct _CrowPanedChild CrowPanedChild;

struct _CrowPanedChild {
	CrowChild parent;
	gboolean resize;
	gboolean shrink;
};

GType crow_paned_child_get_type(void);
void crow_paned_child_init(CrowPanedChild * child);

// CrowNotebookChild -----------------------------------------------------------

#define CROW_TYPE_NOTEBOOK_CHILD (crow_notebook_child_get_type())
#define CROW_NOTEBOOK_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_NOTEBOOK_CHILD, CrowNotebookChild))
#define CROW_IS_NOTEBOOK_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_NOTEBOOK_CHILD))

typedef struct _CrowNotebookChild CrowNotebookChild;

struct _CrowNotebookChild {
	CrowChild parent;
	const char * tabLabel;
	GtkWidget * tabLabelWidget;
	const char * menuLabel;
	GtkWidget * menuLabelWidget;
	CrowPackOptions packOptions;
	gboolean detachable;
	gboolean reorderable;
};

GType crow_notebook_child_get_type(void);
void crow_notebook_child_init(CrowNotebookChild * child);

// CrowAssistantChild ----------------------------------------------------------

#define CROW_TYPE_ASSISTANT_CHILD (crow_assistant_child_get_type())
#define CROW_ASSISTANT_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_ASSISTANT_CHILD, CrowAssistantChild))
#define CROW_IS_ASSISTANT_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_ASSISTANT_CHILD))

typedef struct _CrowAssistantChild CrowAssistantChild;

struct _CrowAssistantChild {
	CrowChild parent;
	GtkAssistantPageType pageType;
	gboolean complete;
	const char * title;
	const char * headerImage;
	const char * sidebarImage;
};

GType crow_assistant_child_get_type(void);
void crow_assistant_child_init(CrowAssistantChild * child);

// CrowTableChild --------------------------------------------------------------

#define CROW_TYPE_TABLE_CHILD (crow_table_child_get_type())
#define CROW_TABLE_CHILD(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_TABLE_CHILD, CrowTableChild))
#define CROW_IS_TABLE_CHILD(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_TABLE_CHILD))

typedef struct _CrowTableChild CrowTableChild;

struct _CrowTableChild {
	CrowChild parent;
	int paddingX;
	int paddingY;
	gboolean xExpand;
	gboolean xFill;
	gboolean xShrink;
	gboolean yExpand;
	gboolean yFill;
	gboolean yShrink;
};

GType crow_table_child_get_type(void);
void crow_table_child_init(CrowTableChild * child);

// -----------------------------------------------------------------------------

typedef CrowChild * PCrowChild;

gboolean ContainerIsEmpty(GtkContainer * container);

gboolean GetExpand(CrowPackOptions packOptions);
gboolean GetFill(CrowPackOptions packOptions);
GtkAttachOptions GetAttachOptions(gboolean expand, gboolean fill, gboolean shrink);

GtkWidget * CreatePlaceholder(void);
gboolean ExposePlaceholder(GtkWidget * widget, GdkEventExpose * event, gpointer data);

void ContainerFixedSet(GtkFixed * fixed, PtrArray * children);
void ContainerLayoutSet(GtkLayout * layout, PtrArray * children);
void ContainerBinSet(GtkBin * bin, PtrArray * children);
void ContainerScrolledWindowSet(GtkScrolledWindow * scroll, PtrArray * children);
void ContainerDialogSet(GtkDialog * dialog, PtrArray * children);
void ContainerInfoBarSet(GtkInfoBar * widget, PtrArray * children);
void ContainerPanedSet(GtkPaned * paned, PtrArray * children);
void ContainerBoxSet(GtkBox * box, PtrArray * children, int capS, int capE);
void ContainerButtonBoxSet(GtkButtonBox * box, PtrArray * children, int capS, int capE);
void ContainerNotebookSet(GtkNotebook * notebook, PtrArray * children, int capacity);
void ContainerAssistantSet(GtkAssistant * assistant, PtrArray * children, int capacity);
void ContainerTableSet(GtkTable * table, PtrArray * children, int capX, int capY);

#endif
