/*
 *  GuiLoader, a GuiXml loader library
 *  Copyright (c) 2006-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#ifndef GUILOADER_GUILOADER_H
#define GUILOADER_GUILOADER_H

#include <gtk/gtk.h>

#define GUILOADER_VERSION (2*1000000 + 19*1000 + 0)

#ifdef _WIN32
	#ifdef GUILOADER_LIBRARY
		#define GUILOADER_API __declspec(dllexport)
	#else
		#define GUILOADER_API __declspec(dllimport)
	#endif
#else
	#define GUILOADER_API __attribute__ ((visibility("default")))
#endif

#define GUIXML_DECL              \
  {                              \
    const char data[]=

#define GUIXML_LOAD(loader)      \
    ;                            \
    gui_loader_load_from_string(loader, data, sizeof(data)-1); \
  }

#define GUIXML_(SSS)

G_BEGIN_DECLS

#define GUI_TYPE_LOADER (gui_loader_get_type())
#define GUI_LOADER(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), GUI_TYPE_LOADER, GuiLoader))
#define GUI_LOADER_CLASS(meta) (G_TYPE_CHECK_CLASS_CAST((meta), GUI_TYPE_LOADER, GuiLoaderClass))
#define GUI_IS_LOADER(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), GUI_TYPE_LOADER))
#define GUI_IS_LOADER_CLASS(meta) (G_TYPE_CHECK_CLASS_TYPE((meta), GUI_TYPE_LOADER))
#define GUI_LOADER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), GUI_TYPE_LOADER, GuiLoaderClass))

typedef struct _GuiLoader GuiLoader;
typedef struct _GuiLoaderClass GuiLoaderClass;
typedef struct _GuiLoaderInit GuiLoaderInit;
typedef struct _GuiConnector GuiConnector;
typedef struct _GuiConnectInfo GuiConnectInfo;

typedef GtkWidget * (*GuiCustomConstructorFunc)(const char * type, const char * meta, const char * text, gpointer data);

struct _GuiLoader {
	GObject parent;
};

struct _GuiLoaderClass {
	GObjectClass parent;
	GObjectClass * parent_class;
	GuiLoaderInit * init;
};

struct _GuiConnectInfo {
	GObject * object;
	const char * signame;
	gboolean after;
};

GUILOADER_API int guiloader_version(void);

GUILOADER_API void guiloader_enter_check(const char * msg);
GUILOADER_API void guiloader_leave(void);

GUILOADER_API GType gui_loader_get_type(void);
GUILOADER_API GuiLoader * gui_loader_new(void);
GUILOADER_API void gui_loader_set_preview_mode(GuiLoader * loader, gboolean preview);
GUILOADER_API void gui_loader_set_take_objects(GuiLoader * loader, gboolean take);
GUILOADER_API void gui_loader_add_custom_constructor(GuiLoader * loader, const char * type, GuiCustomConstructorFunc func, gpointer data, GDestroyNotify destroy);
GUILOADER_API void gui_loader_load_from_string(GuiLoader * loader, const char * data, int size);
GUILOADER_API gboolean gui_loader_load_from_file(GuiLoader * loader, const char * filename, GError ** error);
GUILOADER_API void gui_loader_clear(GuiLoader * loader);
GUILOADER_API GQueue * gui_loader_query_connectors(GuiLoader * loader, const char * name, int count);
GUILOADER_API void gui_loader_get_connect_info(GuiLoader * loader, const GuiConnector * connector, GuiConnectInfo * info);
GUILOADER_API void gui_loader_connect_signals(GuiLoader * loader, const char * name, GCallback func, gpointer userdata, int count);
GUILOADER_API gpointer gui_loader_bind_object(GuiLoader * loader, const char * name);

#define guiloader_enter() guiloader_enter_check(NULL)
#define gui_loader_connect_signal(loader, name, func, userdata) gui_loader_connect_signals(loader, name, func, userdata, 1)

G_END_DECLS

#endif
