/*
 *  GuiLoader, a GuiXml loader library
 *  Copyright (c) 2006-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#ifndef GUILOADER_PROXIES_H
#define GUILOADER_PROXIES_H

#include "guiloader_impl.h"

void RegisterProxies(GuiLoaderInit * init);
void RegisterGType(GType type);

void GObjectProxy(Proxy * proxy);
void PropertyGObjectSignals(PropertyFuncData * data);

// CrowRelationEntry -----------------------------------------------------------

#define CROW_TYPE_RELATION_ENTRY (crow_relation_entry_get_type())
#define CROW_RELATION_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_RELATION_ENTRY, CrowRelationEntry))
#define CROW_IS_RELATION_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_RELATION_ENTRY))

typedef struct _CrowRelationEntry CrowRelationEntry;

struct _CrowRelationEntry {
	GObject parent;
	AtkRelationType relation;
	GtkWidget * target;
};

GType crow_relation_entry_get_type(void);
void crow_relation_entry_init(CrowRelationEntry * entry);

// -----------------------------------------------------------------------------

void CrowRelationEntryProxy(Proxy * proxy);
void PropertyCrowRelationEntryRelation(PropertyFuncData * data);
void PropertyCrowRelationEntryTarget(PropertyFuncData * data);

void GtkWidgetProxy(Proxy * proxy);
void GtkWidgetInit(GObject * object, Node * node);
void PropertyGtkWidgetSizeRequest(PropertyFuncData * data);
void PropertyGtkWidgetTooltip(PropertyFuncData * data);
void PropertyGtkWidgetEvents(PropertyFuncData * data);
void PropertyGtkWidgetAtkName(PropertyFuncData * data);
void PropertyGtkWidgetAtkDescription(PropertyFuncData * data);
void PropertyGtkWidgetAtkRelations(PropertyFuncData * data);
void GtkWidgetAddReceivesDefaultProperty(Proxy * proxy);
void GtkWidgetAddCanFocusProperty(Proxy * proxy);
void GtkWidgetAddCanDefaultProperty(Proxy * proxy);
void GtkWidgetAddAppPaintableProperty(Proxy * proxy);
void GtkWidgetAddEventProperties(Proxy * proxy);
void GtkWidgetAddOrientationProperty(Proxy * proxy);

void GtkContainerProxyBase(Proxy * proxy);

void GtkContainerProxy(Proxy * proxy);
void GtkContainerSetChildrenPropertyFunc(Proxy * proxy, PropertyFunc func);
gboolean GtkContainerVerifyChild(GtkWidget * container, GtkWidget * child);
void PropertyGtkContainerFocusChain(PropertyFuncData * data);

void CrowChildProxy(Proxy * proxy);
void PropertyCrowChildWidget(PropertyFuncData * data);
void CrowChildAddIndexProperty(Proxy * proxy);
void PropertyCrowChildIndex(PropertyFuncData * data);

void GtkBinProxy(Proxy * proxy);
void PropertyGtkBinChildren(PropertyFuncData * data);

void GtkAlignmentProxy(Proxy * proxy);

void GtkFrameProxy(Proxy * proxy);
void PropertyGtkFrameLabelWidgetSet(PropertyFuncData * data);
void PropertyGtkFrameLabel(PropertyFuncData * data);

void GtkAspectFrameProxy(Proxy * proxy);
void GtkAspectFrameInit(GObject * object, Node * node);

void GtkExpanderProxy(Proxy * proxy);
void GtkExpanderInit(GObject * object, Node * node);
void PropertyGtkExpanderLabel(PropertyFuncData * data);

void GtkScrolledWindowProxy(Proxy * proxy);
void PropertyGtkScrolledWindowChildren(PropertyFuncData * data);
void PropertyGtkScrolledWindowAutoViewport(PropertyFuncData * data);

void GtkHandleBoxProxy(Proxy * proxy);
void GtkHandleBoxInit(GObject * object, Node * node);

void GtkViewportProxy(Proxy * proxy);

void CrowBoxChildProxy(Proxy * proxy);
void PropertyCrowBoxChildPack(PropertyFuncData * data);
void PropertyCrowBoxChildOptions(PropertyFuncData * data);
void PropertyCrowBoxChildPadding(PropertyFuncData * data);

void GtkBoxProxy(Proxy * proxy);
void PropertyGtkBoxChildren(PropertyFuncData * data);
void PropertyGtkBoxCapacity(PropertyFuncData * data);

void CrowButtonBoxChildProxy(Proxy * proxy);
void PropertyCrowButtonBoxChildSecondary(PropertyFuncData * data);

void GtkButtonBoxProxy(Proxy * proxy);
void PropertyGtkButtonBoxChildren(PropertyFuncData * data);
void PropertyGtkButtonBoxCapacity(PropertyFuncData * data);

void CrowPanedChildProxy(Proxy * proxy);
void PropertyCrowPanedChildResize(PropertyFuncData * data);
void PropertyCrowPanedChildShrink(PropertyFuncData * data);

void GtkPanedProxy(Proxy * proxy);
void PropertyGtkPanedChildren(PropertyFuncData * data);

void CrowNotebookChildProxy(Proxy * proxy);
void PropertyCrowNotebookChildTabLabel(PropertyFuncData * data);
void PropertyCrowNotebookChildTabLabelWidget(PropertyFuncData * data);
void PropertyCrowNotebookChildMenuLabel(PropertyFuncData * data);
void PropertyCrowNotebookChildMenuLabelWidget(PropertyFuncData * data);
void PropertyCrowNotebookChildPackOptions(PropertyFuncData * data);
void PropertyCrowNotebookChildDetachable(PropertyFuncData * data);
void PropertyCrowNotebookChildReorderable(PropertyFuncData * data);

void GtkNotebookProxy(Proxy * proxy);
void PropertyGtkNotebookChildren(PropertyFuncData * data);
void PropertyGtkNotebookCapacity(PropertyFuncData * data);
void PropertyGtkNotebookPage(PropertyFuncData * data);
void PropertyGtkNotebookActionWidgetStart(PropertyFuncData * data);
void PropertyGtkNotebookActionWidgetEnd(PropertyFuncData * data);

void CrowTableChildProxy(Proxy * proxy);
void PropertyCrowTableChildCell(PropertyFuncData * data);
void PropertyCrowTableChildCellSpan(PropertyFuncData * data);
void PropertyCrowTableChildPaddingX(PropertyFuncData * data);
void PropertyCrowTableChildPaddingY(PropertyFuncData * data);
void PropertyCrowTableChildXExpand(PropertyFuncData * data);
void PropertyCrowTableChildXFill(PropertyFuncData * data);
void PropertyCrowTableChildXShrink(PropertyFuncData * data);
void PropertyCrowTableChildYExpand(PropertyFuncData * data);
void PropertyCrowTableChildYFill(PropertyFuncData * data);
void PropertyCrowTableChildYShrink(PropertyFuncData * data);

void GtkTableProxy(Proxy * proxy);
GObject * GtkTableCreate(GuiLoader * loader, Node * node);
void PropertyGtkTableChildren(PropertyFuncData * data);
void PropertyGtkTableCapacity(PropertyFuncData * data);

void CrowXYChildProxy(Proxy * proxy);
void PropertyCrowXYChildXY(PropertyFuncData * data);

void GtkAdjustmentProxy(Proxy * proxy);

void GtkWindowProxyBase(Proxy * proxy);
void PropertyGtkWindowBaseDefaultSize(PropertyFuncData * data);
void PropertyGtkWindowBaseAccelGroups(PropertyFuncData * data);

void GtkWindowProxy(Proxy * proxy);
GObject * GtkWindowCreate(GuiLoader * loader, Node * node);
void GtkWindowConnectDeleteEvent(GObject * window);
gboolean GtkWindowOnDeleteEvent(GObject * window);
void PropertyGtkWindowDefault(PropertyFuncData * data);

void CrowAssistantChildProxy(Proxy * proxy);
void PropertyCrowAssistantChildPageType(PropertyFuncData * data);
void PropertyCrowAssistantChildComplete(PropertyFuncData * data);
void PropertyCrowAssistantChildTitle(PropertyFuncData * data);
void PropertyCrowAssistantChildHeaderImage(PropertyFuncData * data);
void PropertyCrowAssistantChildSidebarImage(PropertyFuncData * data);

void GtkAssistantProxy(Proxy * proxy);
GObject * GtkAssistantCreate(GuiLoader * loader, Node * node);
void GtkAssistantSetPage(GtkAssistant * widget, gpointer data);
void PropertyGtkAssistantChildren(PropertyFuncData * data);
void PropertyGtkAssistantCapacity(PropertyFuncData * data);
void PropertyGtkAssistantPage(PropertyFuncData * data);
void PropertyGtkAssistantButtons(PropertyFuncData * data);

// CrowDialogButtonEntry -------------------------------------------------------

#define CROW_TYPE_DIALOG_BUTTON_ENTRY (crow_dialog_button_entry_get_type())
#define CROW_DIALOG_BUTTON_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_DIALOG_BUTTON_ENTRY, CrowDialogButtonEntry))
#define CROW_IS_DIALOG_BUTTON_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_DIALOG_BUTTON_ENTRY))

typedef struct _CrowDialogButtonEntry CrowDialogButtonEntry;

struct _CrowDialogButtonEntry {
	GObject parent;
	GtkWidget * widget;
	GtkResponseType response;
	gboolean secondary;
};

GType crow_dialog_button_entry_get_type(void);
void crow_dialog_button_entry_init(CrowDialogButtonEntry * entry);

// -----------------------------------------------------------------------------

void CrowDialogButtonEntryProxy(Proxy * proxy);
void PropertyCrowDialogButtonEntryWidget(PropertyFuncData * data);
void PropertyCrowDialogButtonEntryResponse(PropertyFuncData * data);
void PropertyCrowDialogButtonEntrySecondary(PropertyFuncData * data);

void GtkDialogProxy(Proxy * proxy);
void PropertyGtkDialogChildren(PropertyFuncData * data);
void PropertyGtkDialogButtons(PropertyFuncData * data);
void PropertyGtkDialogDefaultResponse(PropertyFuncData * data);
void PropertyGtkDialogAlternativeButtonOrder(PropertyFuncData * data);

// CrowInfoBarButtonEntry ------------------------------------------------------

#define CROW_TYPE_INFO_BAR_BUTTON_ENTRY (crow_info_bar_button_entry_get_type())
#define CROW_INFO_BAR_BUTTON_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_INFO_BAR_BUTTON_ENTRY, CrowInfoBarButtonEntry))
#define CROW_IS_INFO_BAR_BUTTON_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_INFO_BAR_BUTTON_ENTRY))

typedef struct _CrowInfoBarButtonEntry CrowInfoBarButtonEntry;

struct _CrowInfoBarButtonEntry {
	GObject parent;
	GtkWidget * widget;
	GtkResponseType response;
};

GType crow_info_bar_button_entry_get_type(void);
void crow_info_bar_button_entry_init(CrowInfoBarButtonEntry * entry);

// -----------------------------------------------------------------------------

void CrowInfoBarButtonEntryProxy(Proxy * proxy);
void PropertyCrowInfoBarButtonEntryWidget(PropertyFuncData * data);
void PropertyCrowInfoBarButtonEntryResponse(PropertyFuncData * data);

void GtkInfoBarProxy(Proxy * proxy);
void PropertyGtkInfoBarChildren(PropertyFuncData * data);
void PropertyGtkInfoBarButtons(PropertyFuncData * data);
void PropertyGtkInfoBarDefaultResponse(PropertyFuncData * data);

void GtkColorSelectionDialogProxy(Proxy * proxy);
void GtkColorSelectionDialogInit(GObject * object, Node * node);
void PropertyGtkColorSelectionDialogColorsel(PropertyFuncData * data);

void GtkFileFilterProxy(Proxy * proxy);
void PropertyGtkFileFilterName(PropertyFuncData * data);
void PropertyGtkFileFilterAddPixbufFormats(PropertyFuncData * data);
void PropertyGtkFileFilterPatterns(PropertyFuncData * data);
void PropertyGtkFileFilterMimeTypes(PropertyFuncData * data);

void GtkFileChooserProxyBase(Proxy * proxy);
void PropertyGtkFileChooserBaseFilter(PropertyFuncData * data);
void PropertyGtkFileChooserBaseFilters(PropertyFuncData * data);

void GtkFileChooserDialogProxy(Proxy * proxy);
GObject * GtkFileChooserDialogCreate(GuiLoader * loader, Node * node);

void GtkFontSelectionDialogProxy(Proxy * proxy);
void PropertyGtkFontSelectionDialogFontSelection(PropertyFuncData * data);

void GtkStatusbarProxy(Proxy * proxy);

void GtkActionGroupProxy(Proxy * proxy);
GObject * GtkActionGroupCreate(GuiLoader * loader, Node * node);
void PropertyGtkActionGroupActions(PropertyFuncData * data);

void GtkActionProxy(Proxy * proxy);
GObject * GtkActionCreate(GuiLoader * loader, Node * node);
void PropertyGtkActionAccelerator(PropertyFuncData * data);
void PropertyGtkActionWidgets(PropertyFuncData * data);

void GtkToggleActionProxy(Proxy * proxy);
GObject * GtkToggleActionCreate(GuiLoader * loader, Node * node);

void GtkRadioActionProxy(Proxy * proxy);
GObject * GtkRadioActionCreate(GuiLoader * loader, Node * node);
void PropertyGtkRadioActionGroup(PropertyFuncData * data);

void GtkUIManagerProxy(Proxy * proxy);
void PropertyGtkUIManagerActionGroups(PropertyFuncData * data);
void PropertyGtkUIManagerUiDefinitions(PropertyFuncData * data);

// CrowUIDefinition ------------------------------------------------------------

#define CROW_TYPE_UI_DEFINITION (crow_ui_definition_get_type())
#define CROW_UI_DEFINITION(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_UI_DEFINITION, CrowUIDefinition))
#define CROW_IS_UI_DEFINITION(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_UI_DEFINITION))

typedef struct _CrowUIDefinition CrowUIDefinition;

struct _CrowUIDefinition {
	GObject parent;
	const char * ui;
};

GType crow_ui_definition_get_type(void);
void crow_ui_definition_init(CrowUIDefinition * definition);

// -----------------------------------------------------------------------------

void CrowUIDefinitionProxy(Proxy * proxy);
void PropertyCrowUIDefinitionUi(PropertyFuncData * data);

void CrowUIElementProxy(Proxy * proxy);
GObject * CrowUIElementCreate(GuiLoader * loader, Node * node);

void GtkMenuBarProxy(Proxy * proxy);

void GtkToolbarProxy(Proxy * proxy);
void PropertyGtkToolbarToolbarStyleSet(PropertyFuncData * data);
void PropertyGtkToolbarIconSizeSet(PropertyFuncData * data);

void GtkToolPaletteProxy(Proxy * proxy);
void PropertyGtkToolPaletteToolbarStyleSet(PropertyFuncData * data);
void PropertyGtkToolPaletteIconSizeSet(PropertyFuncData * data);

// CrowRadioGroup --------------------------------------------------------------

#define CROW_TYPE_RADIO_GROUP (crow_radio_group_get_type())
#define CROW_RADIO_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_RADIO_GROUP, CrowRadioGroup))
#define CROW_IS_RADIO_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_RADIO_GROUP))

typedef struct _CrowRadioGroup CrowRadioGroup;

struct _CrowRadioGroup {
	GObject parent;
	GSList * glist;
	GType gtype;
};

GType crow_radio_group_get_type(void);
void crow_radio_group_init(CrowRadioGroup * group);
gboolean crow_radio_group_verify(CrowRadioGroup * group, char * name, GType type);
void crow_radio_group_add_action(CrowRadioGroup * group, char * name, GtkRadioAction * action);
void crow_radio_group_add_button(CrowRadioGroup * group, char * name, GtkRadioButton * button);

// -----------------------------------------------------------------------------

void CrowRadioGroupProxy(Proxy * proxy);
void GtkMiscProxy(Proxy * proxy);

void GtkLabelProxy(Proxy * proxy);

void GtkButtonProxyBase(Proxy * proxy);

void GtkButtonProxy(Proxy * proxy);
void PropertyGtkButtonStockId(PropertyFuncData * data);

void GtkToggleButtonProxy(Proxy * proxy);

void GtkCheckButtonProxy(Proxy * proxy);

void GtkRadioButtonProxy(Proxy * proxy);
void PropertyGtkRadioButtonGroup(PropertyFuncData * data);

void GtkScaleButtonProxy(Proxy * proxy);
GObject * GtkScaleButtonCreate(GuiLoader * loader, Node * node);
void PropertyGtkScaleButtonIcons(PropertyFuncData * data);

void GtkEntryCompletionProxy(Proxy * proxy);
void PropertyGtkEntryCompletionStrings(PropertyFuncData * data);

void GtkEntryProxy(Proxy * proxy);
void PropertyGtkEntryCompletion(PropertyFuncData * data);
void PropertyGtkEntryInnerBorderSet(PropertyFuncData * data);
void PropertyGtkEntryPrimaryIconTooltip(PropertyFuncData * data);
void PropertyGtkEntrySecondaryIconName(PropertyFuncData * data);
void PropertyGtkEntrySecondaryIconTooltip(PropertyFuncData * data);

void GtkComboBoxProxy(Proxy * proxy);
GObject * GtkComboBoxCreate(GuiLoader * loader, Node * node);
void PropertyGtkComboBoxStrings(PropertyFuncData * data);

void GtkComboBoxEntryProxy(Proxy * proxy);
GObject * GtkComboBoxEntryCreate(GuiLoader * loader, Node * node);
void PropertyGtkComboBoxEntryText(PropertyFuncData * data);
void PropertyGtkComboBoxEntryEntry(PropertyFuncData * data);

void GtkSpinButtonProxy(Proxy * proxy);

void GtkRangeProxy(Proxy * proxy);
void GtkRangeProxyInit(GObject * object, Node * node);

// CrowScaleMarkEntry ----------------------------------------------------------

#define CROW_TYPE_SCALE_MARK_ENTRY (crow_scale_mark_entry_get_type())
#define CROW_SCALE_MARK_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_SCALE_MARK_ENTRY, CrowScaleMarkEntry))
#define CROW_IS_SCALE_MARK_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_SCALE_MARK_ENTRY))

typedef struct _CrowScaleMarkEntry CrowScaleMarkEntry;

struct _CrowScaleMarkEntry {
	GObject parent;
	gdouble value;
	GtkPositionType position;
	const char * caption;
};

GType crow_scale_mark_entry_get_type(void);
void crow_scale_mark_entry_init(CrowScaleMarkEntry * entry);

// -----------------------------------------------------------------------------

void CrowScaleMarkEntryProxy(Proxy * proxy);
void PropertyCrowScaleMarkEntryValue(PropertyFuncData * data);
void PropertyCrowScaleMarkEntryPosition(PropertyFuncData * data);
void PropertyCrowScaleMarkEntryCaption(PropertyFuncData * data);

void GtkScaleProxy(Proxy * proxy);
void PropertyGtkScaleMarks(PropertyFuncData * data);

void GtkProgressBarProxy(Proxy * proxy);

void GtkImageProxy(Proxy * proxy);

void GtkSeparatorProxy(Proxy * proxy);

void GtkSizeGroupProxy(Proxy * proxy);
void PropertyGtkSizeGroupWidgets(PropertyFuncData * data);

void GtkTextViewProxy(Proxy * proxy);
void PropertyGtkTextViewText(PropertyFuncData * data);

void GtkIconViewProxy(Proxy * proxy);

void GtkCellViewProxy(Proxy * proxy);

void GtkTreeViewProxy(Proxy * proxy);
void PropertyGtkTreeViewSelectionMode(PropertyFuncData * data);
void PropertyGtkTreeViewSearchEntry(PropertyFuncData * data);

void GtkColorButtonProxy(Proxy * proxy);

void GtkColorSelectionProxy(Proxy * proxy);

void GtkFileChooserButtonProxy(Proxy * proxy);
GObject * GtkFileChooserButtonCreate(GuiLoader * loader, Node * node);

void GtkFileChooserWidgetProxy(Proxy * proxy);
GObject * GtkFileChooserWidgetCreate(GuiLoader * loader, Node * node);

void GtkFontButtonProxy(Proxy * proxy);

void GtkFontSelectionProxy(Proxy * proxy);

void GtkFixedProxy(Proxy * proxy);
void PropertyGtkFixedChildren(PropertyFuncData * data);

void GtkLayoutProxy(Proxy * proxy);
void PropertyGtkLayoutChildren(PropertyFuncData * data);
void PropertyGtkLayoutSize(PropertyFuncData * data);

void GtkScrollbarProxy(Proxy * proxy);

void GtkEventBoxProxy(Proxy * proxy);

void GtkDrawingAreaProxy(Proxy * proxy);

void GtkCalendarProxy(Proxy * proxy);

void GtkArrowProxy(Proxy * proxy);

// CrowCustom ------------------------------------------------------------------

#define CROW_TYPE_CUSTOM (crow_custom_get_type())
#define CROW_CUSTOM(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), CROW_TYPE_CUSTOM, CrowCustom))
#define CROW_CUSTOM_CLASS(meta) (G_TYPE_CHECK_CLASS_CAST((meta), CROW_TYPE_CUSTOM, CrowCustomClass))
#define CROW_IS_CUSTOM(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), CROW_TYPE_CUSTOM))
#define CROW_IS_CUSTOM_CLASS(meta) (G_TYPE_CHECK_CLASS_TYPE((meta), CROW_TYPE_CUSTOM))
#define CROW_CUSTOM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), CROW_TYPE_CUSTOM, CrowCustomClass))

typedef struct _CrowCustom CrowCustom;
typedef struct _CrowCustomClass CrowCustomClass;

struct _CrowCustom {
	GtkDrawingArea parent;
};

struct _CrowCustomClass {
	GtkDrawingAreaClass parent;
};

GType crow_custom_get_type(void);
void crow_custom_class_init(CrowCustomClass * meta);
GtkWidget * crow_custom_new(void);
void crow_custom_size_request(GtkWidget * widget, GtkRequisition * requisition);
int crow_custom_expose(GtkWidget * widget, GdkEventExpose * event);

// -----------------------------------------------------------------------------

void CrowCustomProxy(Proxy * proxy);
GObject * CrowCustomCreate(GuiLoader * loader, Node * node);

void GtkStatusIconProxy(Proxy * proxy);
GObject * GtkStatusIconCreate(GuiLoader * loader, Node * node);
void PropertyGtkStatusIconTooltip(PropertyFuncData * data);
void PropertyGtkStatusIconName(PropertyFuncData * data);

void GtkLinkButtonProxy(Proxy * proxy);
GObject * GtkLinkButtonCreate(GuiLoader * loader, Node * node);

void GtkRecentFilterProxy(Proxy * proxy);
void PropertyGtkRecentFilterName(PropertyFuncData * data);
void PropertyGtkRecentFilterAddPixbufFormats(PropertyFuncData * data);
void PropertyGtkRecentFilterPatterns(PropertyFuncData * data);
void PropertyGtkRecentFilterMimeTypes(PropertyFuncData * data);
void PropertyGtkRecentFilterApplications(PropertyFuncData * data);
void PropertyGtkRecentFilterGroups(PropertyFuncData * data);
void PropertyGtkRecentFilterAge(PropertyFuncData * data);

void GtkRecentChooserProxyBase(Proxy * proxy);
void GtkRecentChooserBaseInit(GObject * object, Node * node);
void PropertyGtkRecentChooserBaseFilter(PropertyFuncData * data);
void PropertyGtkRecentChooserBaseFilters(PropertyFuncData * data);

void GtkRecentChooserWidgetProxy(Proxy * proxy);
void GtkRecentChooserDialogProxy(Proxy * proxy);

void GtkRecentActionProxy(Proxy * proxy);
GObject * GtkRecentActionCreate(GuiLoader * loader, Node * node);

void GtkSpinnerProxy(Proxy * proxy);

#endif
